
////////////////////////////////////////////////////////
//// AUTHOR: YangYunLong1988
//// SOURCE: https://github.com/YangYunLong1988/w5500
////////////////////////////////////////////////////////


module w5500 #(
                parameter   IP_ADDR     = {32'h0},
                parameter   MAC_ADDR    = {48'h0},
                parameter   GW_ADDR     = {32'h0},
                parameter   SN_ADDR     = {32'h0},
                parameter   PORT        = {16'h0}
            )
            (  
                input clk,
                input rst_n,
                input  miso,
                output sclk,
                output mosi,
                output sel,
                output reg w5500_rst
              );

    wire locked;
    wire [7:0] rx_data_1;
    wire [15:0] rx_data_2;
    wire  tx_ready;
    wire  rx_ready;
    wire  rx_flag;
    wire [7:0] buff_8;
    
    reg tx_en;
    reg rx_en;
    reg [71:0] tx_data;
    reg [2:0] tx_len;
    reg [1:0] rx_len;
    reg [15:0] rx_size;
    reg [7:0] mem [0:99];



    //////~~~required?~~~//////
    //localparam   DEST_PORT       = {8'h17,8'h70};                                // Port: 6000
    //localparam   DEST_IP         = {8'd192, 8'd168, 8'd10, 8'd142};              // IP: 192.168.10.142 (my computer)



    localparam	CLKSPI          = 9'd500;   // SPI clock speed 50meg
    localparam	S_RX_SIZE       = 16'd2048; // RIO's 16'd64 ??

    //Register localparam definition
    localparam 	PHYCFGR         = 16'h002e;
    localparam	VDM	            = 8'h00;    // Unfixed length data length
    localparam	RWB_READ	    = 8'h00;    // Read register data
    localparam 	RWB_WRITE	    = 8'h04;    // Write data to register
    localparam	COMMON_R	    = 8'h00;
    localparam	MR	            = 16'h0000;
    localparam	RST	            = 8'h80;
    localparam	GAR	            = 16'h0001;
    localparam	SUBR	        = 16'h0005;
    localparam	SHAR	        = 16'h0009;
    localparam	SIPR	        = 16'h000f;
    localparam	Sn_RXBUF_SIZE	= 16'h001e;
    localparam	Sn_TXBUF_SIZE	= 16'h001f;
    localparam	RTR	            = 16'h0019;
    localparam	RCR             = 16'h001b;
    localparam	Sn_MSSR         = 16'h0012;
    localparam	Sn_PORT         = 16'h0004;
    localparam	Sn_DPORTR       = 16'h0010;
    localparam	Sn_DIPR         = 16'h000c;
    localparam	Sn_MR           = 16'h0000;
    localparam	MR_TCP          = 8'h01;
    localparam	MR_UDP          = 8'h02;
    localparam	Sn_CR           = 16'h0001;
    localparam	OPEN            = 8'h01;
    localparam	Sn_SR           = 16'h0003;
    localparam	CLOSE           = 8'h10;
    localparam	CONNECT         = 8'h04;
    localparam	SOCK_INIT       = 8'h13;
    localparam	SIR             = 16'h0017;
    localparam	Sn_IR           = 16'h0002;
    localparam	Sn_RX_RSR       = 16'h0026;
    localparam	Sn_RX_RD        = 16'h0028;	
    localparam	S0_INT          = 8'h01;
    localparam	IR_CON          = 8'h01;
    localparam	IR_DISCON       = 8'h02;
    localparam	IR_SEND_OK      = 8'h10;
    localparam	IR_RECV         = 8'h04;
    localparam	IR_TIMEOUT      = 8'h08;
    localparam	S_CONN		    = 8'h02;
    localparam	S_TRANSMITOK	= 8'h02;
    localparam	S_RECEIVE		= 8'h01;
    localparam	RECV		    = 8'h40;

    //W5500 read and write frame combination
    localparam	Read_PHYCFGR	= {PHYCFGR,(VDM | RWB_READ | COMMON_R),48'h000000000000};
    localparam	SoftWare_RST	= {MR,(VDM | RWB_WRITE | COMMON_R),RST,40'h0000000000};
    localparam	Set_Gateway	    = {GAR,(VDM | RWB_WRITE | COMMON_R),GW_ADDR,16'h0000};
    localparam	Set_Mask	    = {SUBR,(VDM | RWB_WRITE | COMMON_R),SN_ADDR,16'h0000};
    localparam	Set_PHYADDR		= {SHAR,(VDM | RWB_WRITE | COMMON_R),MAC_ADDR};
    localparam	Set_IPADDR	    = {SIPR,(VDM | RWB_WRITE | COMMON_R),IP_ADDR,16'h0000};
    localparam	Set_TXBUFF	    = {Sn_TXBUF_SIZE,(VDM | RWB_WRITE | 8'h08),8'h02,40'h0000000000};
    localparam	Set_RXBUFF	    = {Sn_RXBUF_SIZE,(VDM | RWB_WRITE | 8'h08),8'h02,40'h0000000000};
    localparam	Set_RETXTIME	= {RTR,(VDM | RWB_WRITE | COMMON_R),8'h07,8'hd0,32'h00000000};
    localparam	Set_TXTIMES	    = {RCR,(VDM | RWB_WRITE | COMMON_R),8'h08,40'h0000000000};

    localparam	Set_FP          = {Sn_MSSR,(VDM | RWB_WRITE | 8'h08),8'h05,8'hb4,32'h00000000};
    localparam	Set_PORT0       = {Sn_PORT,(VDM | RWB_WRITE | 8'h08),PORT,32'h00000000};

    //~~~required?~~~
    //localparam  Set_DESPORT0    = {Sn_DPORTR,(VDM | RWB_WRITE | 8'h08),DEST_PORT,32'h00000000};	    // Destination port number:  6000 
    //localparam  Set_DESIP0      = {Sn_DIPR,(VDM | RWB_WRITE | 8'h08),DEST_IP,16'h0000};	            // Destination IP: 192.168.10.142 (my computer)

    localparam	Set_UDP         = {Sn_MR,(VDM | RWB_WRITE | 8'h08),MR_UDP,40'h0000000000};
    localparam	Set_OPEN        = {Sn_CR,(VDM | RWB_WRITE | 8'h08),OPEN,40'h0000000000};
    localparam	Read_SR         = {Sn_SR,(VDM | RWB_READ | 8'h08),48'h000000000000};
    localparam	Set_CLOSE       = {Sn_CR,(VDM | RWB_WRITE | 8'h08),CLOSE,40'h0000000000};
    localparam	Set_CONN        = {Sn_CR,(VDM | RWB_WRITE | 8'h08),CONNECT,40'h0000000000};

    localparam	Set_READSIR     = {SIR,(VDM | RWB_READ | COMMON_R),48'h000000000000};
    localparam	Set_READSNIR    = {Sn_IR,(VDM | RWB_READ | 8'h08),48'h000000000000};

    localparam	Set_RX_RSR      = {Sn_RX_RSR,(VDM | RWB_READ | 8'h08),48'h000000000000};
    localparam	Set_RX_RD       = {Sn_RX_RD,(VDM | RWB_READ | 8'h08),48'h000000000000};


    // SPI module
    SPI	SPI(	
            .clk(clk),
            .rst_n(rst_n),
            .sclk(sclk),
            .mosi(mosi),
            .miso(miso),
            .rx_data_1(rx_data_1),
            .rx_data_2(rx_data_2),
            .tx_data(tx_data),
            .tx_en(tx_en),
            .rx_en(rx_en),
            .tx_len(tx_len),
            .rx_len(rx_len),
            .rx_size(rx_size),
            .tx_ready(tx_ready),
            .rx_ready(rx_ready),
            .rx_flag(rx_flag),
            .sel(sel),
            .buff_8(buff_8)
            );

    //W5500 configuration is divided into several state machines
    //1. Hardware reset state machine
    //2. W5500 module initialization configuration state machine
    //3. W5500 port initialization configuration state machine
    //4. Data processing state machine, constantly querying the data of the status register to determine the current state of the W5500 module
    //External loop state machine
    reg [2:0]   W5500_STATE;
    localparam	IDLE	=	3'd0;
    localparam	HARD_RST	=	3'd1;
    localparam	W5500_INIT	=	3'd2;
    localparam	W5500_SOCKET_INIT	=	3'd3;
    localparam	SOCKET_WHILE	=	3'd4;
    //The following three states are placed under SOCKET_WHILE
    reg [2:0]   SOCKET_STATE;
    localparam	SOCKET_IDLE	=	3'd0;
    localparam	SOCKET_CONNECT	=	3'd1;
    localparam	W5500_STATUS_CHECK 	=	3'd2;
    localparam	W5500_READ_MEM	=	3'd3;
    localparam	W5500_WRITE_MEM	=	3'd4;	//Write w5500 cache
    //1、Hardware reset state machine
    reg [2:0]   Hardware_State;
    localparam	Hard_IDLE	 =	 3'd0;
    localparam	Hard_RST_LOW 	= 	3'd1;
    localparam	Hard_DELAY50 	=	 3'd2;
    localparam	Hard_RST_HIGH 	= 	3'd3;
    localparam	Hard_DELAY200 	= 	3'd4;
    localparam	Hard_WAIT_CONNECT 	= 	3'd5;
    //2、W5500 module initialization state machine
    reg [3:0]	W5500_INIT_State;
    localparam	W5500_INIT_IDLE	=	4'd0;
    localparam	W5500_INIT_SOFTWARE_RST	=	4'd1;
    localparam	W5500_INIT_DELAY10	=	4'd2;
    localparam   W5500_INIT_SET_GATEWAY = 4'd3; //Set the gateway
    localparam   W5500_INIT_SET_MASK = 4'd4; //Set the subnet mask
    localparam   W5500_INIT_SET_PHYADDR = 4'd5; //Set the local physical address
    localparam   W5500_INIT_SET_IPADDR = 4'd6; //Set the local IP address
    localparam   W5500_INIT_SET_RXBUFF = 4'd7; //Set the receive buffer size
    localparam   W5500_INIT_SET_TXBUFF = 4'd8; //Set the send buffer size
    localparam   W5500_INIT_SET_RETXTIME = 4'd9; //Set the retransmission time
    localparam   W5500_INIT_SET_TXTIMES = 4'd10; //Set the number of retransmissions
    //3、W5500socket initialization state machine
    reg [2:0]	SOCKET_INIT_State;
    localparam	SOCKET_INIT_IDLE	=	3'd0;
    localparam	SOCKET_INIT_FP		=	3'd1;
    localparam	SOCKET_INIT_PORT0	=	3'd2;
    localparam	SOCKET_INIT_DESPORT0	=	3'd3;
    localparam	SOCKET_INIT_DESIP0	= 3'd4;
    //4、SOCKET connection establishment state machine
    reg [2:0]	SOCKET_CONN_State;
    localparam	SOCKET_CONN_IDLE	=	3'd0;
    localparam	SOCKET_CONN_UDP		=	3'd1;
    localparam	SOCKET_CONN_OPEN	=	3'd2;
    localparam	SOCKET_CONN_DELAY5	=	3'd3;
    localparam	SOCKET_CONN_READSR	=	3'd4;
    localparam	SOCKET_CONN_CLOSE	=	3'd5;
    localparam	SOCKET_CONN_SETCONN	=	3'd6; 
    localparam	SOCKET_CONN_READSNR	=	3'd7; // Read the contents of the status register to verify that the connection is set successfully
    //5、SOCKET status check state machine
    reg [2:0] SOCKET_CHACK_State;
    localparam	SOCKET_CHACK_IDLE	=	3'd0;
    localparam	SOCKET_CHACK_READSIR	=	3'd1;
    localparam	SOCKET_CHACK_READSNIR	=	3'd2;
    localparam	SOCKET_CHACK_WRSNIR		=	3'd3;
    localparam	SOCKET_CHACK_IFIR		=	3'd4;
    localparam	SOCKET_CHACK_CLOSE	=	3'd5;
    //6、W5500_READ_MEM reads the state machine of the data in the cache
    reg [2:0] READ_MEM_State;
    localparam	READ_MEM_IDLE		=	3'd0;
    localparam	READ_MEM_READ_RX_RSR	=	3'd1;
    localparam	READ_MEM_READ_RX_RD		=	3'd2;
    localparam	READ_MEM_REC_DATA		=	3'd3;
    localparam	READ_MEM_WR_RX_RD	=	3'd4;
    localparam	READ_MEM_WR_RECV		=	3'd5;
    localparam	READ_MEM_FULL_1			=	3'd6;
    localparam	READ_MEM_TRANS			=	3'd7;
    //The state machine for sending data, i.e. writing data to the W5500 cache
    reg [2:0] WRITE_MEM_State;
    localparam	WRITE_MEM_IDLE		=	3'd0;

    //Delay counter used in this state machine
    //50ms delay counter
    reg [23:0] Delay50_1;
    always @(posedge clk or negedge rst_n)
        if (!rst_n)	Delay50_1 <= 24'd0;
        else if (Hardware_State == Hard_DELAY50)	Delay50_1 <= Delay50_1 + 1'b1;

    //Delay 200ms counter
    reg [23:0] Delay200_1;
    always @(posedge clk or negedge rst_n)
        if (!rst_n)	Delay200_1 <= 24'd0;
        else if (Hardware_State == Hard_DELAY200)	Delay200_1 <= Delay200_1 + 1'b1;
        else Delay200_1 <= 24'd0;

    //Delay 10ms counter
    reg [23:0] Delay10_1;
    always @(posedge clk or negedge rst_n)
        if (!rst_n)	Delay10_1 <= 24'd0;
        else if (W5500_INIT_State == W5500_INIT_DELAY10)		Delay10_1 <= Delay10_1 + 1'b1;
        else Delay10_1 <= 24'd0;

    //Delay 5ms counter
    reg [23:0] Delay5_1;
    always @(posedge clk or negedge rst_n)
        if (!rst_n)	Delay5_1 <= 24'd0;
        else if (SOCKET_CONN_State == SOCKET_CONN_DELAY5)	Delay5_1 <= Delay5_1 + 1'b1;
        else Delay5_1 <= 24'd0;


    //reg [2:0] cnt_RST;
    reg [2:0] cnt_MASK;
    reg [2:0] cnt_PHYADDR;
    reg [2:0] cnt_IPADDR;
    reg [2:0] cnt_RXBUFF;
    reg [2:0] cnt_TXBUFF;
    reg [2:0] cnt_RETXTIME;
    reg [2:0] cnt_TXTIMES;
    reg [2:0] cnt_FP;
    reg [2:0] cnt_PORT0;
    //reg [2:0] cnt_DESPORT0; //~~~required?~~~
    //reg [2:0] cnt_DESIP0; //~~~required?~~~
    reg [2:0] cnt_UDP;
    reg [2:0] cnt_OPEN;
    //reg [2:0] cnt_CLOSE;
    //reg [2:0] cnt_SETCONN;
    reg [2:0] cnt_READSIR;
    reg [2:0] cnt_READSNIR;
    reg [2:0] cnt_SOCKET_CLOSE;
    reg [2:0] cnt_RX_RSR;
    reg [2:0] cnt_RX_RD;
    reg [2:0] cnt_REC_DATA;
    reg [2:0] cnt_WR_RX_RD	;
    reg [2:0] cnt_WR_RECV;
    reg [2:0] cnt_FULL_1;
    reg [2:0] cnt_SETCONN;
    reg [2:0] cnt_READSNR;	
    reg [2:0] cnt_WRSNIR;

    always @(posedge clk or negedge rst_n)
        if (!rst_n)	
            begin
                //cnt_RST <= 3'd0;
                cnt_MASK <= 3'd0;
                cnt_PHYADDR <= 3'd0;
                cnt_IPADDR <= 3'd0;
                cnt_RXBUFF <= 3'd0;
                cnt_TXBUFF <= 3'd0;
                cnt_RETXTIME <= 3'd0;
                cnt_TXTIMES <= 3'd0;
                cnt_FP <= 3'd0;
                cnt_PORT0 <= 3'd0;
                //cnt_DESPORT0 <= 3'd0; //~~~required?~~~
                //cnt_DESIP0 <= 3'd0; //~~~required?~~~
                cnt_UDP<= 3'd0;
                cnt_OPEN <= 3'd0;
                //cnt_CLOSE <= 3'd0;
                //cnt_SETCONN <= 3'd0;
                cnt_READSIR	<= 3'd0;
                cnt_READSNIR <= 3'd0;
                cnt_SOCKET_CLOSE <= 3'd0;
                cnt_RX_RSR	<= 3'd0;
                cnt_RX_RD  <= 3'd0;
                cnt_WR_RX_RD	<=	3'd0;
                cnt_WR_RECV		<=	3'd0;
                cnt_REC_DATA	<=	3'd0;
                cnt_FULL_1	<= 3'd0;
                cnt_SETCONN <= 3'd0;
                cnt_READSNR <= 3'd0;
                cnt_WRSNIR <= 3'd0;
            end
        else
            begin 
                // if (W5500_INIT_State == W5500_INIT_SOFTWARE_RST && cnt_RST < 3'd7)	cnt_RST <= cnt_RST + 1'b1;
                // else 	cnt_RST <= cnt_RST;
                if (W5500_INIT_State == W5500_INIT_SET_MASK && cnt_MASK < 3'd7)	cnt_MASK <= cnt_MASK + 1'b1;
                else 	cnt_MASK <= cnt_MASK;
                if (W5500_INIT_State == W5500_INIT_SET_PHYADDR && cnt_PHYADDR < 3'd7)	  cnt_PHYADDR <= cnt_PHYADDR + 1'b1;
                else 	cnt_PHYADDR <= cnt_PHYADDR;			
                if (W5500_INIT_State == W5500_INIT_SET_IPADDR && cnt_IPADDR < 3'd7)	cnt_IPADDR <= cnt_IPADDR + 1'b1;
                else 	cnt_IPADDR <= cnt_IPADDR;			
                if (W5500_INIT_State == W5500_INIT_SET_RXBUFF && cnt_RXBUFF < 3'd7)	cnt_RXBUFF <= cnt_RXBUFF + 1'b1;
                else 	cnt_RXBUFF <= cnt_RXBUFF;	
                if (W5500_INIT_State == W5500_INIT_SET_TXBUFF && cnt_TXBUFF < 3'd7)	cnt_TXBUFF <= cnt_TXBUFF + 1'b1;
                else 	cnt_TXBUFF <= cnt_TXBUFF;				
                if (W5500_INIT_State == W5500_INIT_SET_RETXTIME && cnt_RETXTIME < 3'd7)	cnt_RETXTIME <= cnt_RETXTIME + 1'b1;
                else 	cnt_RETXTIME <= cnt_RETXTIME;
                if (W5500_INIT_State == W5500_INIT_SET_TXTIMES && cnt_TXTIMES < 3'd7)	cnt_TXTIMES <= cnt_TXTIMES + 1'b1;//改动
                else 	cnt_TXTIMES <= cnt_TXTIMES;
                
                if (SOCKET_INIT_State == SOCKET_INIT_FP && cnt_FP < 3'd7)		cnt_FP <= cnt_FP + 1'b1;
                else cnt_FP <= cnt_FP;
                if (SOCKET_INIT_State == SOCKET_INIT_PORT0 && cnt_PORT0 < 3'd7)		cnt_PORT0 <= cnt_PORT0 + 1'b1;
                else cnt_PORT0 <= cnt_PORT0;

//~~~required?~~~
//                if (SOCKET_INIT_State == SOCKET_INIT_DESPORT0 && cnt_DESPORT0 < 3'd7)		cnt_DESPORT0 <= cnt_DESPORT0 + 1'b1;
//                else cnt_DESPORT0 <= cnt_DESPORT0;
//                if (SOCKET_INIT_State == SOCKET_INIT_DESIP0 && cnt_DESIP0 < 3'd7)		cnt_DESIP0 <= cnt_DESIP0 + 1'b1;
//                else cnt_DESIP0 <= cnt_DESIP0;
                
                if (SOCKET_CONN_State == SOCKET_CONN_UDP && cnt_UDP < 3'd7)		cnt_UDP <= cnt_UDP + 1'b1;
                else if (SOCKET_CONN_State == SOCKET_CONN_UDP && cnt_UDP == 3'd7)	cnt_UDP <= cnt_UDP;
                else cnt_UDP <= 3'd0;
                
                if (SOCKET_CONN_State == SOCKET_CONN_OPEN && cnt_OPEN < 3'd7)		cnt_OPEN <= cnt_OPEN + 1'b1;
                else if (SOCKET_CONN_State == SOCKET_CONN_OPEN && cnt_OPEN == 3'd7)	cnt_OPEN <= cnt_OPEN;
                else cnt_OPEN <= 3'd0;
                
                if (SOCKET_CONN_State == SOCKET_CONN_SETCONN && cnt_SETCONN < 3'd7)		cnt_SETCONN <= cnt_SETCONN + 1'b1;
                else if (SOCKET_CONN_State == SOCKET_CONN_SETCONN && cnt_SETCONN == 3'd7)	cnt_SETCONN <= cnt_SETCONN;
                else cnt_SETCONN <= 3'd0;			
                
                if (SOCKET_CONN_State == SOCKET_CONN_READSNR && cnt_READSNR < 3'd7)		cnt_READSNR <= cnt_READSNR + 1'b1;
                else if (SOCKET_CONN_State == SOCKET_CONN_READSNR && cnt_READSNR == 3'd7)	cnt_READSNR <= cnt_READSNR;
                else cnt_READSNR <= 3'd0;				
                
                // if (SOCKET_CONN_State == SOCKET_CONN_CLOSE && cnt_CLOSE < 3'd7)		cnt_CLOSE <= cnt_CLOSE + 1'b1;
                // else if (SOCKET_CONN_State == SOCKET_CONN_CLOSE && cnt_CLOSE == 3'd7) cnt_CLOSE <= cnt_CLOSE;
                // else cnt_CLOSE <=3'd0;
                
                // if (SOCKET_CONN_State == SOCKET_CONN_SETCONN && cnt_SETCONN < 3'd7)		cnt_SETCONN <= cnt_SETCONN + 1'b1;
                // else if (SOCKET_CONN_State == SOCKET_CONN_SETCONN && cnt_SETCONN == 3'd7) cnt_SETCONN <= cnt_SETCONN;	
                // else 	cnt_SETCONN <= 3'd0;	
                //////This is something you need to think about!!!!!!!

                if (SOCKET_CHACK_State == SOCKET_CHACK_READSIR && cnt_READSIR < 3'd7)		cnt_READSIR <= cnt_READSIR + 1'b1;
                else if (SOCKET_CHACK_State == SOCKET_CHACK_READSIR  && cnt_READSIR == 3'd7)  cnt_READSIR <= cnt_READSIR;
                else cnt_READSIR <= 3'd0;
                
                if (SOCKET_CHACK_State == SOCKET_CHACK_READSNIR && cnt_READSNIR < 3'd7)		cnt_READSNIR <= cnt_READSNIR + 1'b1;
                else if (SOCKET_CHACK_State == SOCKET_CHACK_READSNIR  && cnt_READSNIR == 3'd7)  cnt_READSNIR <= cnt_READSNIR;
                else cnt_READSNIR <= 3'd0;
                
                if (SOCKET_CHACK_State == SOCKET_CHACK_WRSNIR && cnt_WRSNIR < 3'd7)		cnt_WRSNIR <= cnt_WRSNIR + 1'b1;
                else if (SOCKET_CHACK_State == SOCKET_CHACK_WRSNIR  && cnt_WRSNIR == 3'd7)  cnt_WRSNIR <= cnt_WRSNIR;
                else cnt_WRSNIR <= 3'd0;			
                
                if (SOCKET_CHACK_State == SOCKET_CHACK_CLOSE && cnt_SOCKET_CLOSE < 3'd7)		cnt_SOCKET_CLOSE <= cnt_SOCKET_CLOSE + 1'b1;
                else if (SOCKET_CHACK_State == SOCKET_CHACK_CLOSE  && cnt_SOCKET_CLOSE == 3'd7)  cnt_SOCKET_CLOSE <= cnt_SOCKET_CLOSE;
                else cnt_SOCKET_CLOSE <= 3'd0;
                
                if (READ_MEM_State == READ_MEM_READ_RX_RSR && cnt_RX_RSR < 3'd7)		cnt_RX_RSR <= cnt_RX_RSR + 1'b1;
                else if (READ_MEM_State == READ_MEM_READ_RX_RSR  && cnt_RX_RSR == 3'd7)  cnt_RX_RSR <= cnt_RX_RSR;
                else cnt_RX_RSR <= 3'd0;			
                
                if (READ_MEM_State == READ_MEM_READ_RX_RD && cnt_RX_RD < 3'd7)		cnt_RX_RD <= cnt_RX_RD + 1'b1;
                else if (READ_MEM_State == READ_MEM_READ_RX_RD  && cnt_RX_RD == 3'd7)  cnt_RX_RD <= cnt_RX_RD;
                else cnt_RX_RD <= 3'd0;			

                if (READ_MEM_State == READ_MEM_WR_RECV && cnt_WR_RECV < 3'd7)		cnt_WR_RECV <= cnt_WR_RECV + 1'b1;
                else if (READ_MEM_State == READ_MEM_WR_RECV  && cnt_WR_RECV == 3'd7)  cnt_WR_RECV <= cnt_WR_RECV;
                else cnt_WR_RECV <= 3'd0;			

                if (READ_MEM_State == READ_MEM_REC_DATA && cnt_REC_DATA < 3'd7)		cnt_REC_DATA <= cnt_REC_DATA + 1'b1;
                else if (READ_MEM_State == READ_MEM_REC_DATA  && cnt_REC_DATA == 3'd7)  cnt_REC_DATA <= cnt_REC_DATA;
                else cnt_REC_DATA <= 3'd0;	

                if (READ_MEM_State == READ_MEM_WR_RX_RD && cnt_WR_RX_RD < 3'd7)		cnt_WR_RX_RD <= cnt_WR_RX_RD + 1'b1;
                else if (READ_MEM_State == READ_MEM_WR_RX_RD  && cnt_WR_RX_RD == 3'd7)  cnt_WR_RX_RD <= cnt_WR_RX_RD;
                else cnt_WR_RX_RD <= 3'd0;	

                if (READ_MEM_State == READ_MEM_WR_RECV && cnt_WR_RECV < 3'd7)		cnt_WR_RECV <= cnt_WR_RECV + 1'b1;
                else if (READ_MEM_State == READ_MEM_WR_RECV  && cnt_WR_RECV == 3'd7)  cnt_WR_RECV <= cnt_WR_RECV;
                else cnt_WR_RECV <= 3'd0;		

                if (READ_MEM_State == READ_MEM_FULL_1 && cnt_FULL_1 < 3'd7)		cnt_FULL_1 <= cnt_FULL_1 + 1'b1;
                else if (READ_MEM_State == READ_MEM_FULL_1  && cnt_FULL_1 == 3'd7)  cnt_FULL_1 <= cnt_FULL_1;
                else cnt_FULL_1 <= 3'd0;	
                
            end 

    //Count dozens of clock cycles to wait for the ready signal to be pulled low
    reg [3:0] cnt_10;
    always @(posedge clk or negedge rst_n)
        if (!rst_n)	cnt_10 <= 4'd0;
        else if (tx_en == 1'b1 || rx_en == 1'b1)		
            begin
                if (cnt_10 < 4'd10)	cnt_10 <= cnt_10 +1'b1;
                else cnt_10 <= cnt_10;
            end
        else if (tx_en == 1'b0 && rx_en == 1'b0)  cnt_10 <= 4'd0;
        
    //Capture the rising edge of rx_flag
    reg 	rx_flag_1;
    reg	rx_flag_2;
    reg 	rx_flag_3;
    always @(posedge clk or negedge rst_n)
        if (!rst_n)
            begin
                rx_flag_1 <= 1'b0;
                rx_flag_2 <= 1'b0;
                rx_flag_3 <= 1'b0;
            end
        else
            begin
                rx_flag_1 <= rx_flag;
                rx_flag_2 <= rx_flag_1;
                rx_flag_3 <= rx_flag_2;
            end
    wire rx_flag_pos;
    assign rx_flag_pos = rx_flag_1 & rx_flag_2 & (~rx_flag_3);

    //state machine
    reg [7:0] S0_State;
    reg [7:0] DATA_1;
    //reg [15:0] DATA_2;
    reg [15:0] offset;
    reg [15:0] offset1;
    reg [7:0] IR_State;
    reg [6:0] mem_num;
    reg [15:0] rx_size_r;
    reg [15:0] tx_size_next;
    reg [7:0] S0_Data;
    //~~~~~~~Add definition~~~~~~
    reg [15:0] Back_Len;
    reg [15:0] Back_Wid;
    reg [15:0] Front_Len;
    reg [15:0] Front_Wid;
    reg [15:0] Point_X;
    reg [15:0] Point_Y;
    reg [15:0] Width;
    reg [15:0] Hight;
    reg [7:0] Save;
    reg All_Finish;

    always @(posedge clk or negedge rst_n)
        if (!rst_n) 
            begin
                w5500_rst <= 1'b1;
                S0_State <= 8'd0;
                rx_size <= 16'd0;
                mem_num <= 7'd0;
                rx_size_r <= 16'd0;
                tx_size_next <= 16'd0;
                DATA_1 <= 8'd0;
                offset <= 16'd0;
                offset1 <= 16'd0;
                IR_State <= 8'd0;
                S0_Data <= 8'd0;
                tx_en <= 1'b0;
                rx_en <= 1'b0;
                tx_len <= 3'd0;
                rx_len <= 2'd0;
                tx_data <= 72'd0;
                
                //~~~~~~~Add definition~~~~~~
                Back_Len <= 16'd0;
                Back_Wid <= 16'd0;
                Front_Len <= 16'd0;
                Front_Wid <= 16'd0;
                Point_X <= 16'd0;
                Point_Y <= 16'd0;
                Width <= 16'd0;
                Hight <= 16'd0;
                Save <= 8'd0;
                All_Finish <= 1'b0;
                
                W5500_STATE <= IDLE;
                Hardware_State <= Hard_IDLE;
                W5500_INIT_State <= W5500_INIT_IDLE;
                SOCKET_INIT_State <= SOCKET_INIT_IDLE;
                SOCKET_STATE <= SOCKET_IDLE;
                SOCKET_CONN_State <= SOCKET_CONN_IDLE;
                SOCKET_CHACK_State <= SOCKET_CHACK_IDLE;
                READ_MEM_State <= READ_MEM_IDLE;
            end
        else	
            begin
                case (W5500_STATE)
                    IDLE :
                    begin
                        W5500_STATE <= 	HARD_RST;	
                        //else  W5500_STATE <= IDLE;									
                    end
                    
                    HARD_RST :							//1
                    begin
                        case (Hardware_State)
                            Hard_IDLE :						//1					
                            begin
                                Hardware_State <= Hard_RST_LOW;	
                            end
                            Hard_RST_LOW :					//1
                            begin
                                w5500_rst <= 1'b0;
                                Hardware_State <= Hard_DELAY50;
                            end				
                            Hard_DELAY50 :					//1
                            begin
                                if (Delay50_1 == 24'd2500000)		Hardware_State <= Hard_RST_HIGH;
                                else Hardware_State <= Hard_DELAY50;
                            end				
                            Hard_RST_HIGH :				//1
                            begin
                                w5500_rst <= 1'b1;
                                Hardware_State <= Hard_DELAY200;
                            end				
                            Hard_DELAY200 :				//1
                            begin
                                if (Delay200_1 == 24'd10000000)	 Hardware_State <= Hard_WAIT_CONNECT;										
                                else Hardware_State <= Hard_DELAY200;
                            end
                            Hard_WAIT_CONNECT :					//1
                            begin															
                                if (rx_en == 1'b0) 
                                    begin							
                                        rx_en <= 1'b1;
                                        rx_len <= 2'd1;
                                        tx_data <= Read_PHYCFGR;//Composed of data frame Read_PHYCFGR spi will only execute the data in tx_data, and will only execute in parallel, different from C					
                                    end
                                if (rx_ready == 1'b1 && cnt_10 == 4'd10)				
                                    begin
                                        rx_en <= 1'b0;
                                        if (rx_data_1 & 8'h01)
                                            begin
                                                W5500_STATE <= W5500_INIT;
                                                //rx_en <= 1'b0;
                                                tx_data <= 72'd0;
                                            end
                                        else 
                                            begin
                                                rx_en <= 1'b0;
                                                tx_data <= 72'd0;
                                                Hardware_State <= Hard_DELAY200;											
                                            end									
                                    end
                                else 	Hardware_State <= Hard_WAIT_CONNECT;	
                            end
                        endcase					
                    end
                    
                    W5500_INIT :
                    begin
                        case (W5500_INIT_State)
                            W5500_INIT_IDLE :
                            begin
                                W5500_INIT_State <= W5500_INIT_SOFTWARE_RST;
                            end
                            
                            W5500_INIT_SOFTWARE_RST :				//1		
                            begin
                                //if (cnt_RST == 3'd7) 
                                //begin 
                                    if (tx_en == 1'b0) 
                                        begin
                                            tx_en <= 1'b1;
                                            tx_len <= 3'd1;	
                                            tx_data <= SoftWare_RST;
                                        end
                                    if (tx_ready == 1'b1 && cnt_10 == 4'd10)//tx_ready Sending completed
                                        begin
                                            tx_en <= 1'b0;
                                            tx_data <= 72'd0;
                                            W5500_INIT_State <= W5500_INIT_DELAY10;
                                        end
                                    else W5500_INIT_State <= W5500_INIT_SOFTWARE_RST;
                                //end
                            end
                            
                            W5500_INIT_DELAY10 :		//1
                            begin
                                if (Delay10_1 == 24'd500000)	
                                    begin
                                        W5500_INIT_State <= W5500_INIT_SET_GATEWAY;
                                    end
                                else  W5500_INIT_State <= W5500_INIT_DELAY10;
                            end
                            
                            W5500_INIT_SET_GATEWAY	:			//1
                            begin
                                if (tx_en == 1'b0)
                                    begin
                                        tx_en <= 1'b1;
                                        tx_len <= 3'd4;
                                        tx_data <= Set_Gateway;					
                                    end
                                if (tx_ready == 1'b1 && cnt_10 == 4'd10) //spi.v gives tx_ready to indicate that the transmission is complete, cnt_10 delay, the interval between the two is too short, lengthen it
                                    begin
                                        tx_en <= 1'b0;
                                        tx_data <= 72'd0;
                                        W5500_INIT_State <= W5500_INIT_SET_MASK;
                                    end
                                else	W5500_INIT_State <= W5500_INIT_SET_GATEWAY;
                            end
                            
                            W5500_INIT_SET_MASK:			//1
                            begin
                                if (cnt_MASK == 3'd7)
                                begin
                                    tx_en <= 1'b1;
                                    tx_len <= 3'd4;
                                    tx_data <= Set_Mask;
                                    if (tx_ready == 1'b1 && cnt_10 == 4'd10)
                                        begin
                                            tx_en <= 1'b0;
                                            tx_data <= 72'd0;
                                            W5500_INIT_State <= W5500_INIT_SET_PHYADDR;
                                        end
                                    else	W5500_INIT_State <= W5500_INIT_SET_MASK;
                                end
                                    else W5500_INIT_State <= W5500_INIT_SET_MASK;
                            end
                            
                            W5500_INIT_SET_PHYADDR :			//1
                            begin
                                if (cnt_PHYADDR == 3'd7)
                                begin
                                    tx_en <= 1'b1;
                                    tx_len <= 3'd6;	
                                    tx_data <= Set_PHYADDR;
                                    if (tx_ready == 1'b1 && cnt_10 == 4'd10)
                                        begin
                                            tx_en <= 1'b0;
                                            tx_data <= 72'd0;
                                            W5500_INIT_State <= W5500_INIT_SET_IPADDR;
                                        end
                                    else	W5500_INIT_State <= W5500_INIT_SET_PHYADDR;
                                end
                                else	W5500_INIT_State <= W5500_INIT_SET_PHYADDR;
                            end
                            
                            W5500_INIT_SET_IPADDR :			//w5500 IP
                            begin
                                if (cnt_IPADDR == 3'd7)
                                begin
                                    tx_en <= 1'b1;
                                    tx_len <= 3'd4;
                                    tx_data <= Set_IPADDR;
                                    if (tx_ready == 1'b1 && cnt_10 == 4'd10)
                                        begin
                                            tx_en <= 1'b0;
                                            tx_data <= 72'd0;
                                            W5500_INIT_State <= W5500_INIT_SET_RXBUFF;
                                        end
                                    else	W5500_INIT_State <= W5500_INIT_SET_IPADDR;
                                end
                                else	W5500_INIT_State <= W5500_INIT_SET_IPADDR;
                            end
                            
                            W5500_INIT_SET_RXBUFF :		//1
                            begin
                                if (cnt_RXBUFF == 3'd7)
                                begin
                                    tx_en <= 1'b1;
                                    tx_len <= 3'd1;
                                    tx_data <= Set_RXBUFF;
                                    if (tx_ready == 1'b1 && cnt_10 == 4'd10)
                                        begin
                                            tx_en <= 1'b0;
                                            tx_data <= 72'd0;
                                            W5500_INIT_State <= W5500_INIT_SET_TXBUFF;
                                        end		
                                    else	W5500_INIT_State <= W5500_INIT_SET_RXBUFF;
                                end
                                else	W5500_INIT_State <= W5500_INIT_SET_RXBUFF;
                            end
                            
                            W5500_INIT_SET_TXBUFF :			//1
                            begin
                                if (cnt_TXBUFF == 3'd7)
                                begin
                                    tx_en <= 1'b1;
                                    tx_len <= 3'd1;
                                    tx_data <= Set_TXBUFF;
                                    if (tx_ready == 1'b1 && cnt_10 == 4'd10)
                                        begin
                                            tx_en <= 1'b0;
                                            tx_data <= 72'd0;
                                            W5500_INIT_State <= W5500_INIT_SET_RETXTIME;
                                        end
                                    else 	W5500_INIT_State <= W5500_INIT_SET_TXBUFF;									
                                end
                                else	W5500_INIT_State <= W5500_INIT_SET_TXBUFF;	
                            end
                            
                            W5500_INIT_SET_RETXTIME :			//1
                            begin
                                if (cnt_RETXTIME == 3'd7)
                                begin
                                    tx_en <= 1'b1;
                                    tx_len <= 3'd2;
                                    tx_data <= Set_RETXTIME;
                                    if (tx_ready == 1'b1 && cnt_10 == 4'd10)
                                        begin
                                            tx_en <= 1'b0;
                                            tx_data <= 72'd0;
                                            W5500_INIT_State <= W5500_INIT_SET_TXTIMES;
                                        end
                                    else	W5500_INIT_State <= W5500_INIT_SET_RETXTIME;
                                end
                                else	W5500_INIT_State <= W5500_INIT_SET_RETXTIME;
                            end
                            
                            W5500_INIT_SET_TXTIMES :			//1
                            begin
                                if (cnt_TXTIMES == 3'd7)
                                begin
                                    tx_en <= 1'b1;
                                    tx_len <= 3'd1;
                                    tx_data <= Set_TXTIMES;
                                    if (tx_ready == 1'b1 && cnt_10 == 4'd10)
                                        begin
                                            tx_en <= 1'b0;
                                            tx_data <= 72'd0;
                                            W5500_STATE <= W5500_SOCKET_INIT;
                                        end
                                    else	W5500_INIT_State <= W5500_INIT_SET_TXTIMES;
                                end
                                else	W5500_INIT_State <= W5500_INIT_SET_TXTIMES;
                            end												
                        endcase
                    end
                    
                    W5500_SOCKET_INIT :
                    begin
                        case(SOCKET_INIT_State) 				//1
                            SOCKET_INIT_IDLE : begin
                                SOCKET_INIT_State <= SOCKET_INIT_FP;
                            end
                            
                            SOCKET_INIT_FP: begin			//1
                                if (cnt_FP == 3'd7)
                                begin
                                    tx_en <= 1'b1;
                                    tx_len <= 3'd2;
                                    tx_data <= Set_FP;
                                    if (tx_ready == 1'b1 && cnt_10 == 4'd10)
                                        begin
                                            tx_en <= 1'b0;
                                            tx_data <= 72'd0;
                                            SOCKET_INIT_State <= SOCKET_INIT_PORT0;
                                        end
                                    else SOCKET_INIT_State <= SOCKET_INIT_FP;
                                end
                                else	SOCKET_INIT_State <= SOCKET_INIT_FP;
                            end
                            
                            SOCKET_INIT_PORT0: begin			//Setting the Port
                                if (cnt_PORT0 == 3'd7)
                                begin
                                    tx_en <= 1'b1;
                                    tx_len <= 3'd2;
                                    tx_data <= Set_PORT0;
                                    if (tx_ready == 1'b1 && cnt_10 == 4'd10)
                                        begin
                                            tx_en <= 1'b0;
                                            tx_data <= 72'd0;
                                            SOCKET_INIT_State <= SOCKET_INIT_DESPORT0;
                                        end
                                    else SOCKET_INIT_State <= SOCKET_INIT_PORT0;
                                end
                                else	SOCKET_INIT_State <= SOCKET_INIT_PORT0;
                            end

                            //~~~required?~~~
                            /*                          
                            SOCKET_INIT_DESPORT0 : begin				//Set the destination port (My Computer)
                                if (cnt_DESPORT0 == 3'd7)
                                begin
                                    tx_en <= 1'b1;
                                    tx_len <= 3'd2;
                                    tx_data <= Set_DESPORT0;
                                    if (tx_ready == 1'b1 && cnt_10 == 4'd10)
                                        begin
                                            tx_en <= 1'b0;
                                            tx_data <= 72'd0;
                                            SOCKET_INIT_State <= SOCKET_INIT_DESIP0;
                                        end
                                    else SOCKET_INIT_State <= SOCKET_INIT_DESPORT0;
                                end
                                else	SOCKET_INIT_State <= SOCKET_INIT_DESPORT0;
                            end
                            
                            SOCKET_INIT_DESIP0 : begin					
                                if (cnt_DESIP0 == 3'd7)
                                begin
                                    tx_en <= 1'b1;
                                    tx_len <= 3'd4;
                                    tx_data <= Set_DESIP0;
                                    if (tx_ready == 1'b1 && cnt_10 == 4'd10)
                                        begin
                                            tx_en <= 1'b0;
                                            tx_data <= 72'd0;
                                            W5500_STATE <= SOCKET_WHILE;										
                                            SOCKET_INIT_State <= SOCKET_INIT_IDLE;
                                        end
                                    else SOCKET_INIT_State <= SOCKET_INIT_DESIP0;
                                end
                                else	SOCKET_INIT_State <= SOCKET_INIT_DESIP0;
                            end		
                            */						


                        endcase
                    end
            SOCKET_WHILE:		
                begin
                case (SOCKET_STATE)
                    SOCKET_IDLE:
                    begin
                        if (S0_State == 8'd0)
                            SOCKET_STATE <= SOCKET_CONNECT;
                        else	SOCKET_STATE <= W5500_STATUS_CHECK;
                    end				
                                    
                    SOCKET_CONNECT:
                    begin
                        case (SOCKET_CONN_State)
                            SOCKET_CONN_IDLE: begin			//1
                                    SOCKET_CONN_State <= SOCKET_CONN_UDP;						
                            end
                            
                            SOCKET_CONN_UDP: begin				//Setting up TCP communication
                                if (cnt_UDP == 3'd7)
                                begin
                                    tx_en <= 1'b1;
                                    tx_len <= 3'd1;
                                    tx_data <= Set_UDP;
                                    if (tx_ready == 1'b1 && cnt_10 == 4'd10)
                                        begin
                                            tx_en <= 1'b0;
                                            tx_data <= 72'd0;
                                            SOCKET_CONN_State <= SOCKET_CONN_OPEN;										
                                        end
                                    else	SOCKET_CONN_State <= SOCKET_CONN_UDP;
                                end
                                else	SOCKET_CONN_State <= SOCKET_CONN_UDP;
                            end
                            
                            SOCKET_CONN_OPEN: begin			//Opening a socket
                                if (cnt_OPEN == 3'd7)
                                begin
                                    tx_en <= 1'b1;
                                    tx_len <= 3'd1;
                                    tx_data <= Set_OPEN;
                                    if (tx_ready == 1'b1 && cnt_10 == 4'd10) 
                                        begin
                                            tx_en <= 1'b0;
                                            tx_data <= 72'd0;
                                            SOCKET_CONN_State <= SOCKET_CONN_DELAY5;										
                                        end
                                    else	SOCKET_CONN_State <= SOCKET_CONN_OPEN;	
                                end
                                else	SOCKET_CONN_State <= SOCKET_CONN_OPEN;	
                            end
                            
                            SOCKET_CONN_DELAY5: begin
                                if (Delay5_1 == 24'd250000)
                                    SOCKET_CONN_State <= SOCKET_CONN_READSR;
                                else SOCKET_CONN_State <= SOCKET_CONN_DELAY5;
                            end
                            
                            SOCKET_CONN_READSR: begin					//1
                                if (rx_en == 1'b0)
                                    begin
                                        rx_en <= 1'b1;
                                        rx_len <= 2'd1;
                                        tx_data <= Read_SR;	
                                    end	
                                if (rx_ready == 1'b1 && cnt_10 == 4'd10)	
                                    begin
                                        if (rx_data_1 != SOCK_INIT)			//The difference between != and !==
                                            begin
                                                rx_en <= 1'b0;
                                                tx_data <= 72'd0;	
                                                S0_State <= 8'h00;
                                                SOCKET_CONN_State <= SOCKET_CONN_CLOSE;											
                                            end
                                        else 
                                            begin
                                                rx_en <= 1'b0;
                                                tx_data <= 72'd0;								
                                                SOCKET_CONN_State <= SOCKET_CONN_SETCONN;
                                            end
                                    end
                            end
                            
                            SOCKET_CONN_CLOSE: begin			//1
                                if (tx_en == 1'b0)
                                begin
                                    tx_en <= 1'b1;
                                    tx_len <= 3'd1;
                                    tx_data <= Set_CLOSE;							
                                end
                                    if (tx_ready == 1'b1 && cnt_10 == 4'd10)
                                        begin
                                            tx_en <= 1'b0;
                                            tx_data <= 72'd0;
                                            SOCKET_CONN_State <= SOCKET_CONN_IDLE;
                                        end
                                    else SOCKET_CONN_State <= SOCKET_CONN_CLOSE;							
                            end
                            
                            SOCKET_CONN_SETCONN: begin			//1
                                if (cnt_SETCONN == 3'd7)
                                begin
                                    tx_en <= 1'b1;
                                    tx_len <= 3'd1;
                                    tx_data <= Set_CONN;
                                
                                    if (tx_ready == 1'b1 && cnt_10 == 4'd10)
                                        begin
                                            tx_en <= 1'b0;
                                            tx_data <= 72'd0;
                                            S0_State <= 8'h01;		//SOCKET connection successful
                                            //SOCKET_CONN_State <= SOCKET_CONN_READSNR;
                                            SOCKET_STATE <= W5500_STATUS_CHECK;
                                            SOCKET_CONN_State <= SOCKET_CONN_IDLE;
                                        end
                                    else SOCKET_CONN_State <= SOCKET_CONN_SETCONN;		
                                end 
                            end	
    /* 						//Here you can add a very short delay state
                            SOCKET_CONN_READSNR	:begin
                                if (rx_en == 1'b0)
                                    begin
                                        rx_en <= 1'b1;
                                        rx_len <= 2'd1;
                                        tx_data <= Read_SR;	
                                    end	
                                if (rx_ready == 1'b1 && cnt_10 == 4'd10)	
                                    begin
                                        if (rx_data_1 == 8'h17)			//The difference between != and !==
                                            begin
                                                rx_en <= 1'b0;
                                                tx_data <= 72'd0;	
                                                // S0_State <= 8'h00;
                                                SOCKET_STATE <= W5500_STATUS_CHECK;
                                                
                                                // SOCKET_CONN_State <= SOCKET_CONN_IDLE;											
                                            end
                                        else 							
                                                SOCKET_CONN_State <= SOCKET_CONN_READSNR;
                                    end							
                            end */
                            

                            
                        endcase 
                    end				
                    
                    W5500_STATUS_CHECK ://Query the status register to see if there is data
                    begin
                         case (SOCKET_CHACK_State)
                            SOCKET_CHACK_IDLE:begin				//0
                                SOCKET_CHACK_State <= SOCKET_CHACK_READSIR;
                            end
                            
                            SOCKET_CHACK_READSIR	:begin		//1			//Here, the interrupt flag register needs to be read cyclically until data is read. That is, 72 bytes of frame data need to be sent cyclically.							
                                if (rx_en == 1'b0)
                                    begin
                                        rx_en <= 1'b1;
                                        rx_len <= 2'd1;
                                        tx_data <= Set_READSIR;
                                    end
                                if (rx_ready == 1'b1 &&  cnt_10 == 4'd10)
                                    begin
                                        rx_en <= 1'b0;
                                        tx_data <= 72'd0;
                                        if ((rx_data_1 & S0_INT) == S0_INT)									
                                            SOCKET_CHACK_State <= SOCKET_CHACK_READSNIR;									
                                        else SOCKET_CHACK_State <= SOCKET_CHACK_IDLE;			//Return to IDLE state
                                    end
                            end
                            
                            SOCKET_CHACK_READSNIR:begin			//Determine which interrupt
                                if (cnt_READSNIR == 3'd7)
                                begin
                                    rx_en <= 1'b1;
                                    rx_len <= 2'd1;
                                    tx_data <= Set_READSNIR;
                                    if (rx_ready == 1'b1 &&  cnt_10 == 4'd10)
                                        begin
                                            DATA_1 <= rx_data_1;
                                            rx_en <= 1'b0;
                                            tx_data <= 72'd0;
                                            SOCKET_CHACK_State <= SOCKET_CHACK_WRSNIR;										
                                        end
                                    else SOCKET_CHACK_State <= SOCKET_CHACK_READSNIR;
                                end
                            end
                            
                            SOCKET_CHACK_WRSNIR:begin			//Clear the status register						
                                    if (cnt_WRSNIR == 3'd7)
                                    begin
                                            tx_en <= 1'b1;
                                            tx_len <= 3'd1;
                                            tx_data <= {Sn_IR,(VDM | RWB_WRITE | 8'h08),DATA_1,40'h0000000000};
                                    
                                    if (tx_ready == 1'b1 && cnt_10 == 4'd10)
                                        begin
                                            tx_en <= 1'b0;
                                            tx_data <= 72'd0;
                                            SOCKET_CHACK_State <= SOCKET_CHACK_IFIR;
                                        end
                                    else  SOCKET_CHACK_State <= SOCKET_CHACK_WRSNIR;
                                    end
                            end
                            
                            SOCKET_CHACK_IFIR:begin		//Determine what interrupts
                            if (DATA_1 & 8'h1f)
                            begin
                                if (DATA_1 & IR_CON)
                                    begin
                                        S0_State<= S0_State | S_CONN;
                                        SOCKET_CHACK_State <= SOCKET_CHACK_IDLE;
                                    end							
                                 if (DATA_1 & IR_DISCON)
                                    begin
                                        S0_State <= 8'h00;
                                        SOCKET_CHACK_State <= SOCKET_CHACK_CLOSE;
                                    end
                                 if (DATA_1 & IR_SEND_OK)
                                    begin
                                        S0_Data <= S0_Data | S_TRANSMITOK;
                                        SOCKET_CHACK_State <= SOCKET_CHACK_IDLE;
                                    end
                                 if (DATA_1 & IR_RECV)					//Detect that data has been received and start entering the data receiving state machine
                                    begin
                                        S0_Data	 <= S0_Data | S_RECEIVE;
                                        SOCKET_CHACK_State <= SOCKET_CHACK_IDLE;
                                        SOCKET_STATE <= W5500_READ_MEM;
                                    end
                                 if (DATA_1 & IR_TIMEOUT)
                                    begin
                                        S0_State <= 8'd0;
                                        SOCKET_CHACK_State <= SOCKET_CHACK_CLOSE;									
                                    end
                                //else  SOCKET_CHACK_State <= SOCKET_CHACK_IDLE;
                            end
                            else 	SOCKET_CHACK_State <= SOCKET_CHACK_IDLE;
                            end							

                            SOCKET_CHACK_CLOSE:begin						//5
                                if (cnt_SOCKET_CLOSE == 3'd7)
                                begin
                                    tx_en <= 1'b1;
                                    tx_len <= 3'd1;
                                    tx_data <= Set_CLOSE;
                                    if (tx_ready == 1'b1 && cnt_10 == 4'd10)
                                        begin
                                            tx_en <= 1'b0;
                                            tx_data <= 72'd0;
                                            W5500_STATE <= W5500_SOCKET_INIT;	
                                            SOCKET_STATE <= SOCKET_IDLE;
                                            SOCKET_CHACK_State <= SOCKET_CHACK_IDLE;
                                        end
                                    else SOCKET_CHACK_State <= SOCKET_CHACK_CLOSE;
                                end							
                            end
                        endcase	
                    end
                    
                    W5500_READ_MEM :
                    begin
                        case (READ_MEM_State)
                            READ_MEM_IDLE: begin						//0
                                READ_MEM_State <= READ_MEM_READ_RX_RSR;
                            end
                            
                            READ_MEM_READ_RX_RSR: begin				//Confirm the spi data length, sent by the computer
                                if (cnt_RX_RSR == 3'd7)
                                begin
                                    rx_en <= 1'b1;
                                    rx_len <= 2'd2;
                                    tx_data <= Set_RX_RSR;
                                    if (rx_ready == 1'b1 &&  cnt_10 == 4'd10)
                                        begin
                                            rx_en <= 1'b0;
                                            tx_data <= 72'd0;
                                            //rx_size <= rx_data_2;
                                            rx_size_r <= rx_data_2;
                                            if (rx_data_2 == 16'd0)
                                                begin
                                                    SOCKET_STATE <=	W5500_STATUS_CHECK;
                                                    READ_MEM_State <= READ_MEM_IDLE;
                                                end											
                                            else if (rx_data_2 > 16'd1460)	
                                                begin
                                                    rx_size_r <= 16'd1460;
                                                    READ_MEM_State <= READ_MEM_READ_RX_RD;
                                                end
                                            else 	READ_MEM_State <= READ_MEM_READ_RX_RD;
                                            
                                        end
                                end
                            end
                            
                            READ_MEM_READ_RX_RD: begin                      //Confirm the spi data address, sent by the computer
                                if (cnt_RX_RD == 3'd7)
                                begin
                                    rx_en <= 1'b1;
                                    rx_len <= 2'd2;
                                    tx_data <= Set_RX_RD;
                                    if (rx_ready == 1'b1 &&  cnt_10 == 4'd10)
                                        begin
                                            rx_en <= 1'b0;
                                            tx_data <= 72'd0;
                                            offset1 <= rx_data_2;
                                            offset <= (S_RX_SIZE-1) & rx_data_2;
                                            if (((S_RX_SIZE-1) & rx_data_2)+ rx_size_r < S_RX_SIZE)   
                                                begin
                                                    rx_size <= rx_size_r;
                                                    READ_MEM_State <= READ_MEM_REC_DATA;
                                                end
                                            else 
                                                begin
                                                    tx_size_next <= S_RX_SIZE - ((S_RX_SIZE-1) & rx_data_2);
                                                    rx_size <= S_RX_SIZE - ((S_RX_SIZE-1) & rx_data_2);												
                                                    READ_MEM_State <= 	READ_MEM_REC_DATA;										
                                                end
                                        end
                                    else READ_MEM_State <= READ_MEM_READ_RX_RD;
                                end
                                else	READ_MEM_State <= READ_MEM_READ_RX_RD;
                            end
                            
                            READ_MEM_REC_DATA: begin                            //Start fetching data according to address and length
                                if (cnt_REC_DATA == 3'd7)
                                begin
                                    rx_en <= 1'b1;
                                    rx_len <= 2'd0;
                                    tx_data <= {offset,(VDM | RWB_READ | 8'h18),48'h000000000000};
                                    if (rx_flag_pos)
                                        begin
                                            if (mem_num <= 7'd100)              //How to process the received data
                                                begin
                                                    mem [mem_num] <= buff_8;
                                                    mem_num <= mem_num + 1'b1;											
                                                end 
                                        end
                                    if (rx_ready == 1'b1 &&  cnt_10 == 4'd10)
                                        begin
                                            rx_en <= 1'b0;
                                            tx_data <= 72'd0;
                                            if ((offset + rx_size_r) > S_RX_SIZE)	
                                                begin
                                                    READ_MEM_State  <= READ_MEM_FULL_1;
                                                end
                                            else
                                                begin
                                                    offset1 <= offset1 + rx_size_r;
                                                    READ_MEM_State <= READ_MEM_WR_RX_RD;	
                                                end
                                        end
                                end							
                            end
                            
                            READ_MEM_FULL_1: begin
                                if (cnt_FULL_1 == 3'd7)
                                    begin
                                        rx_en <= 1'b1;
                                        rx_len <= 2'd0;		
                                        tx_data <= {16'h0000,(VDM | RWB_READ | 8'h18),48'h000000000000};
                                        rx_size <= rx_size_r - tx_size_next;
                                        if (rx_flag_pos)
                                            begin
                                                if (mem_num <= 7'd100)	
                                                    begin
                                                        mem [mem_num] <= buff_8;
                                                        mem_num <= mem_num + 1'b1;			
                                                    end
                                            end									
                                        if (rx_ready == 1'b1 &&  cnt_10 == 4'd10)
                                            begin
                                                rx_en <= 1'b0;
                                                tx_data <= 72'd0;
                                                offset1 <= offset1 + rx_size_r;
                                                READ_MEM_State <= READ_MEM_WR_RX_RD;											
                                            end
                                    end
                            end 
                            
                            READ_MEM_WR_RX_RD: begin
                                if (cnt_WR_RX_RD == 3'd7)
                                begin
                                    tx_en <= 1'b1;
                                    tx_len <= 3'd2;
                                    tx_data <= {Sn_RX_RD,(VDM | RWB_WRITE | 8'h08),offset1,32'h00000000};
                                    All_Finish <= 1'b0;			//Pull the flag bit low and wait for the data frame to be assembled, then pull it high		
                                    if (tx_ready == 1'b1 &&  cnt_10 == 4'd10)
                                        begin
                                            tx_en <= 1'b0;
                                            tx_data <= 72'd0;
                                            READ_MEM_State <= READ_MEM_WR_RECV;
                                        end
                                    else READ_MEM_State <= READ_MEM_WR_RX_RD;
                                end
                                else	READ_MEM_State <= READ_MEM_WR_RX_RD;
                            end
                            
                            READ_MEM_WR_RECV: begin					//
                                if (cnt_WR_RECV == 3'd7)
                                begin
                                    tx_en <= 1'b1;
                                    tx_len <= 3'd1;
                                    tx_data <= {Sn_CR,(VDM | RWB_WRITE | 8'h08),RECV,40'h0000000000};
                                    if (tx_ready == 1'b1 &&  cnt_10 == 4'd10)
                                        begin
                                            tx_en <= 1'b0;
                                            tx_data <= 72'd0;
                                            //mem_num <= 7'd0;
                                            //SOCKET_STATE <= SOCKET_IDLE;
                                            //READ_MEM_State <= READ_MEM_IDLE;
                                            READ_MEM_State <= READ_MEM_TRANS;
                                        end
                                    else READ_MEM_State <= READ_MEM_WR_RECV;
                                end
                                else		READ_MEM_State <= READ_MEM_WR_RECV;
                            end	

                            READ_MEM_TRANS : begin
                                if (mem_num == 7'd17)
                                begin
                                    Back_Len <= {mem[0],mem[1]};
                                    Back_Wid <= {mem[2],mem[3]};
                                    Front_Len <= {mem[4],mem[5]};
                                    Front_Wid <= {mem[6],mem[7]};
                                    Point_X <= {mem[8],mem[9]};
                                    Point_Y <= {mem[10],mem[11]};
                                    Width <= {mem[12],mem[13]};
                                    Hight <= {mem[14],mem[15]};
                                    Save <= mem[16];
                                    mem_num <= 7'd0;
                                    All_Finish <= 1'b1;			//A frame of data has been completely received, and the flag is pulled high
                                    SOCKET_STATE <= SOCKET_IDLE;
                                    READ_MEM_State <= READ_MEM_IDLE;								
                                end
                                else READ_MEM_State <= READ_MEM_TRANS;
                            end
                endcase		
                    end
                W5500_WRITE_MEM: begin
                
                end
            endcase
            end
        endcase
        end
	
endmodule
